import pandas as pd
import json
from sachima.filter_enum import FilterEnum


def set_sql_params(sql, params):
    """
    set params to sql
    return sql str\n
    for example:
        select {colname1} from {tablename} where {colname2} = '{value}'
    """
    copy_params = {}
    copy_params.update(params)
    # convert dict to tuple for sql
    for k in params:
        if isinstance(copy_params[k], list):
            copy_params[k] = str(tuple(copy_params[k])).replace(",)", ")")
    return sql.format(**copy_params)


class Filter:
    def __init__(self, id, setter, **kw):
        """
        id: str
        setter: tuple
        """
        self.id = id
        self.setter = setter
        self.kw = kw

    def __repr__(self):
        return "Filter(" + self.id + ")"

    def to_json(self, data):
        res = {"id": "", "props": {}}
        res.update(self.kw)
        res["id"] = self.id

        if not isinstance(data, pd.DataFrame):
            raise TypeError(
                "pd.DataFrame expected but get {}".format(str(type(data)))
            )

        # todo: json str from enumn tree improve
        for arg in self.setter:
            if isinstance(arg, FilterEnum.TYPE):
                res["type"] = arg.value
            if isinstance(arg, FilterEnum.PROPS.MODE):  # bug++++
                res["props"].update({"mode": arg.value})
            if isinstance(arg, FilterEnum.PROPS.ALLOWCLEAR):
                res["props"].update({"allowClear": arg.value})
            if isinstance(arg, FilterEnum.PROPS.SIZE):
                res["props"].update({"size": arg.value})
            if isinstance(arg, dict):
                colname = arg.get("option", None)
                if isinstance(colname, str) and colname in data.columns:
                    res.update(
                        {
                            "option": data[colname]
                            .sort_values()
                            .map(lambda x: str(x))
                            .unique()
                            .tolist()
                        }
                    )
                elif colname:
                    res.update(arg)
                else:
                    res["props"].update(arg)
        # print(res)
        return res


def data_wrapper(data):
    """
    data: dict
    return: json str
    return json str to api for frontend \n
    for example:
        antd
    """
    # data["data"]
    # data["filters"]
    if not data:
        return {"columns": ["提示信息"], "dataSource": [{"提示信息": "出现错误请联系管理员"}]}

    # print(data)
    print("changing the data into json str...")
    res = {}
    df = data["data"][0]
    filters = data["filters"]

    if isinstance(df, pd.DataFrame):
        res["controls"] = [f.to_json(df) for f in filters]
        res["columns"] = df.columns.tolist()
        res["dataSource"] = json.loads(
            df.to_json(
                orient="records",
                date_format="iso",
                date_unit="s",
                force_ascii=False,
            )
        )
        print("------------------return api----------------------")
        # print("{{{{{{{{{{{{{{{{{{{")
        # print(res)
        # return json.dumps(res)
        return res
    else:
        raise TypeError("your handler should return pd.DataFrame")



from aenum import Enum, unique, skip


class FilterEnum(Enum):
    @skip
    class TYPE(Enum):
        DATE = "DatePicker"
        DATERANGE = "RangePicker"
        ITEMSELECT = "Select"

    @skip
    class PROPS(Enum):
        @skip
        class SIZE(Enum):
            SMALL = "small"
            BIG = "big"

        @skip
        class MODE(Enum):
            TAGS = "tags"
            MULTIPLE = "multiple"

        @skip
        class ALLOWCLEAR(Enum):
            TRUE = "True"
            FALSE = "False"

        @skip
        class P(Enum):
            A = "A"
            B = "B"



import inspect
import functools
import importlib
import os
import numpy as np
import pandas as pd
import json
from nameko.rpc import rpc, RpcProxy
from sachima.publish import Publisher
from sachima.params import Filter, data_wrapper


def api(platform="superset", isRun=False, **kw_api):
    def wrapper(func):
        @functools.wraps(func)
        def api_called(*args, **kw):

            # 调用对应注册接口
            if platform == "superset":
                fpath = inspect.getfile(func)
                fname = os.path.split(fpath)[1]
                fmod = os.path.splitext(fname)[0]
                Publisher.to_superset(
                    name=kw_api.get("name", fmod),
                    type_=kw_api.get("type_", "RPC"),
                    param=fmod,
                )
            else:
                pass

            # 默认不运行
            if isRun:
                _result = func(*args, **kw)
                return _result

        return api_called

    return wrapper


class Data(object):
    name = "data"

    @rpc
    def get_report(self, params):
        m = importlib.import_module(params["name"])
        return data_wrapper(m.main(params))



import functools
import time
import inspect


def timer(func):
    """Print the runtime of the decorated function"""

    @functools.wraps(func)
    def wrapper_timer(*args, **kwargs):
        start_time = time.perf_counter()  # 1
        value = func(*args, **kwargs)
        end_time = time.perf_counter()  # 2
        run_time = end_time - start_time  # 3
        print(f"Finished {inspect.getfile(func)} in {run_time:.4f} secs")
        return value

    return wrapper_timer


def debug(func):
    """Print the function signature and return value"""

    @functools.wraps(func)
    def wrapper_debug(*args, **kwargs):
        args_repr = [repr(a) for a in args]  # 1
        kwargs_repr = [f"{k}={v!r}" for k, v in kwargs.items()]  # 2
        signature = ", ".join(args_repr + kwargs_repr)  # 3
        print(f"Calling {func.__name__}({signature})")
        value = func(*args, **kwargs)
        print(f"{func.__name__!r} returned {value!r}")  # 4
        return value

    return wrapper_debug


def slow_down(func):
    """Sleep 1 second before calling the function"""

    @functools.wraps(func)
    def wrapper_slow_down(*args, **kwargs):
        time.sleep(1)
        return func(*args, **kwargs)

    return wrapper_slow_down


def singleton(cls):
    """Make a class a Singleton class (only one instance)"""

    @functools.wraps(cls)
    def wrapper_singleton(*args, **kwargs):
        if not wrapper_singleton.instance:
            wrapper_singleton.instance = cls(*args, **kwargs)
        return wrapper_singleton.instance

    wrapper_singleton.instance = None
    return wrapper_singleton


class Averager:
    def __init__(self):
        self.series = []

    def __call__(self, new_value):
        self.series.append(new_value)
        total = sum(self.series)
        return total / len(self.series)


def make_averager():
    count = 0
    total = 0

    def averager(new_value):
        nonlocal count, total  # 自由变量 闭包
        count += 1
        total += new_value
        return total / count

    return averager


def clock(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        t0 = time.time()
        result = func(*args, **kwargs)
        elapsed = time.time() - t0
        name = func.__name__
        arg_lst = []
        if args:
            arg_lst.append(", ".join(repr(arg) for arg in args))
        if kwargs:
            pairs = ["%s=%r" % (k, w) for k, w in sorted(kwargs.items())]
            arg_lst.append(", ".join(pairs))
        arg_str = ", ".join(arg_lst)
        print("[%0.8fs] %s(%s) => %r" % (elapsed, name, arg_str, result))
        return result

    return wrapper


def lengthOfLongestSubstring(s):
    """
    :type s: str
    :rtype: int
    """
    stub = -1
    max_len = 0
    cache = {}

    for i, v in enumerate(s):
        if v in cache and cache[v] > stub:
            stub = cache[v]
            cache[v] = i
        else:
            cache[v] = i
            if i - stub > max_len:
                max_len = i - stub
    return max_len


def tools_extract(df, p, *cols):
    for c in cols:
        filter_list = p.get(c, None)
        if p.get(c, None) == "" or p.get(c, None) is None:
            continue
        if isinstance(p.get(c, None), str):
            filter_list = [filter_list]
        if filter_list:
            print(df)
            print("---------c------")
            print(c)
            print(filter_list)
            df = df[df[c].isin(filter_list)]
    return df



from nameko.standalone.rpc import ClusterRpcProxy

from sanic import Sanic
from sanic.response import json
from sachima import conf

app = Sanic()

CONFIG = conf.get("BROKER")


def sachima_rpc_reports(req):
    with ClusterRpcProxy(CONFIG) as rpc:
        print(req)
        res = rpc.data.get_report(req.json)
        return res


@app.route("/reports", methods=["POST"])
async def sachima(request):
    print(request)
    return json(sachima_rpc_reports(request))
